@@ -805,7 +805,9 @@ class RuleConverter():
                                     '.array())-%s).abs() < R).cast<int>().'\
                                     'sum() != 0) ',
                          'free':'  ((((X_spatial.block(0,0,1,n_ribosomes).array()'\
-                                ')-%s).abs() < R).cast<int>().sum() == 0) ',}
+                                ')-%s).abs() < R).cast<int>().sum() == 0) ',
+                          'rand':'unif(rng)',
+                          'probes':'X_probes'}
 
         #if operators to parse
         self.if_operators = ['<', '>', '==', '!=', '>=', '<=']
@@ -1280,6 +1282,19 @@ class RuleConverter():
                 new_indexes = '.block(0,%s,1,%s)'%(j, k)
 
             newvar = self.keywords[key] + new_indexes
+            
+        if key in ['probes']:
+            if ':' not in index_str:
+                new_indexes = '(' + index_str +')'
+
+            else:
+                if ',' in index_str:
+                    x=1
+                else:
+                    j, k = str(index_str.split(':')[0]), str(index_str.split(':')[1])
+                    new_indexes = '.block(0,%s,1,%s)'%(j, k)
+
+            newvar = self.keywords[key] + new_indexes            
 
         if key in ['free', 'occupied']:
             newvar = self.keywords[key]%index_str
